# This module is copied from PloneHelpCenter product
from AccessControl import ClassSecurityInfo
from Products.CMFQuickInstallerTool.QuickInstallerTool import QuickInstallerTool
from Products.CMFCore.CMFCorePermissions import ManagePortal

PATCH_PREFIX = '_monkey_'

__refresh_module__ = 0


def monkeyPatch(originalClass, patchingClass):
    #print 'monkeyPatch', originalClass.__name__, patchingClass.__name__
    """Monkey patch original class with attributes from new class
       (Swiped from SpeedPack -- thanks, Christian Heimes!)

    * Takes all attributes and methods except __doc__ and __module__ from patching class
    * Safes original attributes as _monkey_name
    * Overwrites/adds these attributes in original class
    """
    for name, newAttr in patchingClass.__dict__.items():
        # don't overwrite doc or module informations
        if name not in ('__doc__', '__module__'):
            # safe the old attribute as __monkey_name if exists
            # __dict__ doesn't show inherited attributes :/
            orig = getattr(originalClass, name, None)
            if orig:
                stored_orig_name = PATCH_PREFIX + name
                stored_orig = getattr(originalClass, stored_orig_name, None)
                # don't double-patch on refresh!
                if stored_orig is None:
                    originalClass.__dict__[stored_orig_name] = orig
            # overwrite or add the new attribute
            originalClass.__dict__[name] = newAttr

from Products.CMFQuickInstallerTool.QuickInstallerTool import not_installed
from Acquisition import aq_parent

class PatchedQuickInstallerTool:
    #
    #   Quick Installer
    #

    security = ClassSecurityInfo()

    security.declareProtected(ManagePortal, 'checkProductIsRestricted')
    def checkProductIsRestricted(self, p):
        """ Check whether product 'p' is allowed for installation.
        Restricted product list is found in property cmf_quickinstaller_restrictions.
        This property should be placed in the Zope root to be modified only
        by the Zope instance manager but not by the CMF/Plone instance managers
        """
        restricted = 0   # not restricted by default
        restrictions = getattr(self.getPhysicalRoot(), 'cmf_quickinstaller_restrictions', ())
        if restrictions:
            site_id = getattr(aq_parent(self), 'id', '')
            if site_id:
                for restriction in restrictions:
                    rproduct, rsites = restriction.split('|')
                    rsites = rsites.split(',')
                    if (rproduct == p) and (site_id not in rsites):
                        restricted = 1
                        break
        return restricted

    security.declareProtected(ManagePortal, 'listInstallableProducts')
    def listInstallableProducts(self,skipInstalled=1):
        """List candidate CMF products for
        installation -> list of dicts with keys:(id,hasError,status)
        """
        pids = self.Control_Panel.Products.objectIds() + not_installed(self)

        pids = [pid for pid in pids if self.isProductInstallable(pid)]

        if skipInstalled:
            installed=[p['id'] for p in self.listInstalledProducts(showHidden=1)]
            pids=[r for r in pids if r not in installed]

        # the following one line is CMFQIProductsRestriction patch
        pids=[r for r in pids if not self.checkProductIsRestricted(r)]

        res=[]
        for r in pids:
            p=self._getOb(r,None)
            if p:
                res.append({'id':r, 'status':p.getStatus(),
                            'hasError':p.hasError()})
            else:
                res.append({'id':r, 'status':'new', 'hasError':0})
        return res


# We don't need to check version. listInstallableProducts method was last changed at
# Tue Feb 17 17:24:29 2004 (adding pacman support)
# see http://svn.plone.org/view/collective?rev=2154&view=rev

import zLOG
"""
from Products.CMFQuickInstallerTool import quickinstaller_globals
from App.Common import package_home
from os.path import join

x = []
QI_VERSION = 'Unknown'
try:
    file = join(package_home(quickinstaller_globals), 'version.txt')
    QI_VERSION = open(file, 'r').read().strip()
    version = QI_VERSION.strip()
    if version.lower().startswith('cmf-'):
        version = version[4:]
    filtered = ''
    for v in version:
        if v in ['0','1','2','3','4','5','6','7','8','9','.']:
            filtered += v
        else:
            break
    x = [int(x) for x in filtered.split('.')]
except IOError:
    # couldnt find file, oh well
    pass
except ValueError:
    # couldnt make sense of the version number
    pass
if x == [1,5,2]:
    zLOG.LOG('CMFQIProductsRestrictions', zLOG.INFO, 'Monkey patching CMFQuickInstallerTool')
    monkeyPatch(QuickInstallerTool, PatchedQuickInstallerTool)
else:
    #raise 'CMFQIProductsRestrictions patch should be used only qith CMFQuickInstallerTool 1.5.2. Please check patched methods'
    zLOG.LOG('CMFQIProductsRestrictions', zLOG.WARN, 'Monkey patch CMFQuickInstallerTool NOT APPLIED - This patch is only for version 1.5.2')
"""
zLOG.LOG('CMFQIProductsRestrictions', zLOG.INFO, 'Monkey patching CMFQuickInstallerTool')
monkeyPatch(QuickInstallerTool, PatchedQuickInstallerTool)
